;
; ------------------------------------------------------------
;
;   Shape Designer
;
;   collectordave  2015
;
; ------------------------------------------------------------

UseSQLiteDatabase()
UsePNGImageDecoder()

Structure DrawObject
  
  Type.s       ;Line Or Arc
  StartX.i     ;Start of line or centre for arc
  StartY.i     ;Start of line or centre for arc
  EndX.i       ;End of line not used for arc
  EndY.i       ;End of line not used for arc
  Radius.i     ;Radius for Arc
  StartAngle.i ;Start Angle for Arc
  EndAngle.i   ;End Angle for Arc
  
EndStructure

Global NewList Elements.DrawObject()

;Dim DrawObjects.DrawObject(0)
Global NewObject.DrawObject

Enumeration FormMenu
  #mnuShapeNew
  #muShapeLoad
  #mnuShapeSave
  #MenuItem_5
  #mnuShapeView
  #mnuShapeTest
  #MenuItem_8
  #mnuShapeExit
EndEnumeration

Enumeration
  #Empty_Image
  #Drawn_Image
  #IMAGE_Color
EndEnumeration

;Control Variables
Global DrawAction.i = 0
Global CurrentColor.i, CurrentMode.s,CurrentOperation.s 
Global SelectedShape_id.i

;Main Form Gadget variables
Global MainWnd.i,drgCanvas.i,btnLoad.i,btnSave.i,btnNew.i,btnCancel.i,cmbGroups.i,cmbShapes.i
Global strStartX.i,strStartY.i,strNewShape.i,strAngle1.i,strAngle2.i,strRadius.i

;Shapes database variables
Global ShapesDB.i,Group_ID.i

Global NewShape.i
Global Shape.s
Global MyColour = RGBA($00,$00,$00,$FF)

Procedure DrawShape()
  
  ;Procedure to draw the current shape
  
  StartVectorDrawing(CanvasVectorOutput(drgCanvas))
  ;Clear Canvas
  DrawVectorImage(ImageID(#Empty_Image), 255)
  ForEach Elements()       ; Process all the elements...
    Select Elements()\Type
      Case "Line"
        MovePathCursor(Elements()\StartX,Elements()\StartY) 
        AddPathLine(Elements()\EndX,Elements()\EndY)
      Case "Arc"
        AddPathCircle(Elements()\StartX,Elements()\StartY,Elements()\Radius,Elements()\StartAngle,Elements()\EndAngle)
    EndSelect
  Next

  StrokePath(0.1) 
  StopVectorDrawing()
  
EndProcedure

Procedure.l OpenShapesDB()

    DatabaseFile$ = GetCurrentDirectory() +"Shapes.spb"
   
   OpenDB.l = OpenDatabase(#PB_Any, DatabaseFile$, "", "")
   
   If OpenDB.l > 0 
     
     ProcedureReturn OpenDB.l
     
   Else

     ProcedureReturn 0
     
   EndIf
   
 EndProcedure
 
Procedure LoadShapescmb()
    
    Group.s = GetGadgetText(cmbGroups)
    
    If group.s = ""
      
      ;No Group Selected
      Group_ID = -1
      ProcedureReturn

    EndIf
    
    If DatabaseQuery(ShapesDB, "SELECT * FROM Groups WHERE Name = '" + Group.s + "';")
      
      FirstDatabaseRow(ShapesDB)
      Group_ID = GetDatabaseLong(ShapesDB, 0)
      FinishDatabaseQuery(ShapesDB)
      
      ;Get the shapes defined in this group
      If DatabaseQuery(ShapesDB, "SELECT * FROM Shapes WHERE Group_ID = " + Str(Group_ID) + ";")
        ClearGadgetItems(cmbShapes)  

        While NextDatabaseRow(ShapesDB)
          AddGadgetItem(cmbShapes, -1, GetDatabaseString(ShapesDB, 1))
        Wend
        FinishDatabaseQuery(ShapesDB)
      EndIf
     
    EndIf
     
EndProcedure
   
Procedure LoadShape(Shape_ID)
    
  If DatabaseQuery(ShapesDB, "SELECT * FROM DrawPoints WHERE Shape_ID = " + StrU(Shape_ID) + ";")

    ClearList(Elements())
                
    While NextDatabaseRow(ShapesDB)
      Select GetDatabaseString(ShapesDB, DatabaseColumnIndex(ShapesDB, "PathType"))
        Case "Line"

          AddElement(Elements())
          Elements()\Type = GetDatabaseString(ShapesDB, DatabaseColumnIndex(ShapesDB, "PathType"))
          Elements()\StartX = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "StartX")) * 5
          Elements()\StartY = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "StartY")) * 5
          Elements()\EndX = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "EndX")) * 5
          Elements()\EndY = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "EndY")) * 5 
 
        Case "Arc"

          AddElement(Elements())
          Elements()\Type = GetDatabaseString(ShapesDB, DatabaseColumnIndex(ShapesDB, "PathType"))
          Elements()\StartX = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "StartX")) * 5
          Elements()\StartY = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "StartY")) * 5
          Elements()\Radius = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "Radius")) * 5
          Elements()\StartAngle = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "StartAngle"))
          Elements()\EndAngle = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "EndAngle"))

      EndSelect
  
    Wend
    
    DrawShape()
        
    FinishDatabaseQuery(ShapesDB)  
    
  EndIf
    
EndProcedure
 
Procedure ProcessAction(x.f, y.f)
  
  Static CentreX.i,CentreY.i,Radius.i,Angle1.i,Angle2.i
  
  If StartVectorDrawing(CanvasVectorOutput(drgCanvas))
    ;Clear Canvas
    DrawVectorImage(ImageID(#Drawn_Image), 255)
  
    Select CurrentMode
        
      Case "Line"
        Select DrawAction
          Case 0
            
            NewObject\StartX = X
            NewObject\StartY = Y
            
          Case 1
            
            MovePathCursor(NewObject\StartX,NewObject\StartY)
            AddPathLine(X,Y)
            StrokePath(0.1)
            
        EndSelect
        
      Case "Arc"
        
        Select DrawAction
          Case 0
            CentreX = X
            CentreY = Y 
          Case 1 ;Start Point Chosen so chose radius
            

            NewObject\Radius = Sqr(Pow((CentreX-X),2) + Pow((CentreY-Y),2))
            VectorSourceColor(RGBA($FF,$00,$00,$FF))
            SetGadgetText(strRadius,Str(NewObject\Radius))
            AddPathCircle(X,Y, NewObject\Radius , 0,360)
            DashPath(1, 5)

          Case 2 ;Start Point And radius now get start Angle
            
            ;Show Radius circle as selected
            VectorSourceColor(RGBA($00,$00,$FF,$FF))
            AddPathCircle(NewObject\StartX,NewObject\StartY, NewObject\Radius , 0,360);,#PB_Path_CounterClockwise)  
            DashPath(1, 5);, #PB_Path_RoundEnd)    
            
            ;Show Chose angle line           
            VectorSourceColor(RGBA($FF,$00,$00,$FF))
            MovePathCursor(NewObject\StartX,NewObject\StartY)
            AddPathLine(x,y)
            DashPath(1, 5)
            
            If x => NewObject\StartX And Y => NewObject\StartY
              If X - NewObject\StartX = 0
                Angle1 = 90
              Else
                Angle1 = Degree(ATan((Y - NewObject\StartY)/(X - NewObject\StartX)))
              EndIf
            ElseIf  x =< NewObject\StartX And Y => NewObject\StartY
              If Y-NewObject\StartY = 0
                Angle1 = 180
              Else
                Angle1 = 90 + Degree(ATan((NewObject\StartX-X)/(Y-NewObject\StartY)))
              EndIf
            ElseIf  x =< NewObject\StartX And Y =< NewObject\StartY
              If X - NewObject\StartX = 0
                Angle1 = 270
              Else
                Angle1 = 180 + Degree(ATan((Y-NewObject\StartY)/(X - NewObject\StartX)))
              EndIf
            ElseIf  x => NewObject\StartX And Y =< NewObject\StartY  
              Angle1 = 270 + Degree(ATan((X - NewObject\StartX)/(NewObject\StartY-Y)))
            EndIf
            
            SetGadgetText(strAngle1,Str(Angle1))
            NewObject\StartAngle = Angle1
            
          Case 3 ;Get End Angle
                        
            ;Show Chose angle line           
            VectorSourceColor(RGBA($FF,$00,$00,$FF))
            MovePathCursor(NewObject\StartX,NewObject\StartY)
            AddPathLine(x,y)
            DashPath(1, 5)
            If x => NewObject\StartX And Y => NewObject\StartY
              If X - NewObject\StartX = 0
                Angle2 = 90
              Else
                Angle2 = Degree(ATan((Y - NewObject\StartY)/(X - NewObject\StartX)))
              EndIf
            ElseIf  x =< NewObject\StartX And Y => NewObject\StartY
              If Y-NewObject\StartY = 0
                Angle2 = 180
              Else
                Angle2 = 90 + Degree(ATan((NewObject\StartX-X)/(Y-NewObject\StartY)))
              EndIf
            ElseIf  x =< NewObject\StartX And Y =< NewObject\StartY
              If X - NewObject\StartX = 0
                Angle2 = 270
              Else
                Angle2 = 180 + Degree(ATan((Y-NewObject\StartY)/(X - NewObject\StartX)))
              EndIf
            ElseIf  x => NewObject\StartX And Y =< NewObject\StartY  
              Angle2 = 270 + Degree(ATan((X - NewObject\StartX)/(NewObject\StartY-Y)))
            EndIf
            SetGadgetText(strAngle2,Str(Angle2))
            NewObject\EndAngle = Angle2
            AddPathCircle(NewObject\StartX,NewObject\StartY, NewObject\Radius , NewObject\StartAngle,Angle2);,#PB_Path_CounterClockwise) 
        EndSelect    
    EndSelect
    VectorSourceColor(MyColour)
    StrokePath(0.1)
    StopVectorDrawing()

  EndIf

EndProcedure

Procedure DrawGrid()
  
  Grid.i = 5

  StartDrawing(CanvasOutput(drgCanvas))

  ;Draw Vertical
  j.i = 0 
  For i = 0 To GadgetWidth(drgCanvas)
    j = j + grid.i
    i = j
    k.i = 0
     For l = 0 To GadgetHeight(drgCanvas)
    k = k + grid.i
    l = k
    If i < OutputWidth() And l < OutputHeight()
      Plot(i,l, RGB(100,100, 100))
      EndIf
    Next l
  Next i

  StopDrawing()

EndProcedure

Procedure NewShape()

  ;Clear Canvas
  StartVectorDrawing(CanvasVectorOutput(drgCanvas))
  DrawVectorImage(ImageID(#Empty_Image), 255)
  StopVectorDrawing()
  
  ;Clear str gadgets
  SetGadgetText(strAngle1,"")
  SetGadgetText(strAngle2,"")
  SetGadgetText(strRadius,"")
  
  ;Clear The Element List
  ClearList(Elements())
         
  ;Hide Shapes Combo
  HideGadget(cmbShapes,1)
  ;Show String gadget
  HideGadget(strNewShape,0)
  CurrentOperation  = "New"
  
EndProcedure

Procedure SaveShape()
  
            
         Select CurrentOperation
             
             Case "New"
               
               ;Check Shape Name Entered
               If GetGadgetText(strNewShape) > ""
       
                 ;Add Shape to database
                 DatabaseUpdate(ShapesDB, "INSERT INTO Shapes (Title,Group_ID) VALUES ('" + GetGadgetText(strNewShape) + "'," + Str(Group_ID) + ");")

                                ;Get selected shapeID
               If DatabaseQuery(ShapesDB, "SELECT * FROM Shapes WHERE Title = '" + GetGadgetText(cmbShapes) + "';") 
             
                 FirstDatabaseRow(ShapesDB)
                 SelectedShape_id = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "Shape_ID"))
                 FinishDatabaseQuery(ShapesDB)
      
                 ;Delete any current drawpoints for this shape
                 DatabaseUpdate(ShapesDB, "DELETE FROM DrawPoints WHERE Shape_ID = " + Str(SelectedShape_id) + " ;")
           
                 ;Save Elements List
                 ForEach Elements()
           
                   DatabaseUpdate(ShapesDB, "INSERT INTO DrawPoints (Shape_ID,PathType,StartX,StartY,EndX,EndY,Radius,StartAngle,EndAngle) VALUES (" + Str(SelectedShape_id) + ",'" + Elements()\Type + "',"  + Str(Elements()\StartX/5) + ","  + Str(Elements()\StartY/5) + "," + Str(Elements()\EndX/5) + "," + Str(Elements()\EndY/5) + "," + Str(Elements()\Radius/5) + "," + Str(Elements()\StartAngle) + "," + Str(Elements()\EndAngle) + ");")
 
                 Next
                 
               EndIf
               
               Else
                 MessageRequester("Save Error","No Shape Name Entered")
                 SetActiveGadget(strNewShape)
               EndIf
               
             Case "Edit"
               
               ;Get selected shapeID
               If DatabaseQuery(ShapesDB, "SELECT * FROM Shapes WHERE Title = '" + GetGadgetText(cmbShapes) + "';") 
             
                 FirstDatabaseRow(ShapesDB)
                 SelectedShape_id = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "Shape_ID"))
                 FinishDatabaseQuery(ShapesDB)
      
                 ;Delete any current drawpoints for this shape
                 DatabaseUpdate(ShapesDB, "DELETE FROM DrawPoints WHERE Shape_ID = " + Str(SelectedShape_id) + " ;")
           
                 ;Save Elements List
                 ForEach Elements()
           
                   DatabaseUpdate(ShapesDB, "INSERT INTO DrawPoints (Shape_ID,PathType,StartX,StartY,EndX,EndY,Radius,StartAngle,EndAngle) VALUES (" + Str(SelectedShape_id) + ",'" + Elements()\Type + "',"  + Str(Elements()\StartX/5) + ","  + Str(Elements()\StartY/5) + "," + Str(Elements()\EndX/5) + "," + Str(Elements()\EndY/5) + "," + Str(Elements()\Radius/5) + "," + Str(Elements()\StartAngle) + "," + Str(Elements()\EndAngle) + ");")
 
                 Next
                 
               EndIf
               
           EndSelect  
  
EndProcedure

CurrentColor = $000000
CreateImage(#IMAGE_Color, 35, 35, 24)
CreateImage(#Drawn_Image, 500,500, 24)

MainWnd = OpenWindow(#PB_Any, x, y, 720, 550, "Shape Designer", #PB_Window_SystemMenu)
CreateMenu(0, WindowID(MainWnd))
MenuTitle("Shape")
MenuItem(#mnuShapeNew, "New")
MenuItem(#muShapeLoad, "Load")
MenuItem(#mnuShapeSave, "Save")
MenuBar()
MenuItem(#mnuShapeView, "View All")
MenuItem(#mnuShapeTest, "Test")
MenuBar()
MenuItem(#mnuShapeExit, "Exit") 
drgCanvas = CanvasGadget(#PB_Any, 10, 10, 505, 505)
cmbGroups = ComboBoxGadget(#PB_Any, 530, 50, 170, 20)
cmbShapes = ComboBoxGadget(#PB_Any, 530, 80, 170, 20)
strNewShape = StringGadget(#PB_Any, 530, 80, 170, 20, "")
btnLoad = ButtonGadget(#PB_Any , 530, 110, 50, 30, "Load")
btnSave = ButtonGadget(#PB_Any, 590, 110, 50, 30, "Save")
btnNew = ButtonGadget(#PB_Any, 650, 110, 50, 30, "New")
btnCancel = ButtonGadget(#PB_Any, 565, 150, 100, 30, "Cancel")
btn_Line = ButtonGadget(#PB_Any, 530, 300, 60, 30, "Line")
btn_Arc = ButtonGadget(#PB_Any, 630, 300, 60, 30, "Arc")
Frame_0 = FrameGadget(#PB_Any, 520, 10, 190, 180, "", #PB_Frame_Single)
Frame_1 = FrameGadget(#PB_Any, 520, 200, 190, 60, "", #PB_Frame_Single)
frmCurrentOperation = FrameGadget(#PB_Any, 520, 270, 190, 240, "", #PB_Frame_Single)
TextGadget(#PB_Any, 620, 230, 20, 20, "Y", #PB_Text_Right)
TextGadget(#PB_Any, 530, 230, 20, 20, "X", #PB_Text_Right)
TextGadget(#PB_Any, 530, 20, 80, 20, "Select Shape")
TextGadget(#PB_Any, 530, 210, 100, 20, "Current Position")
TextGadget(#PB_Any, 530, 280, 110, 20, "Current Operation")
TextGadget(#PB_Any, 530, 340, 60, 20, "Start")
TextGadget(#PB_Any, 620, 340, 60, 20, "End")
TextGadget(#PB_Any, 530, 400, 40, 20, "Angle 1")
TextGadget(#PB_Any, 580, 400, 40, 20, "Angle 2")
TextGadget(#PB_Any, 630, 400, 70, 20, "Radius")
strCurrentX = StringGadget(#PB_Any, 560, 230, 40, 20, "")
strCurrentY = StringGadget(#PB_Any, 650, 230, 40, 20, "") 
strStartX = StringGadget(#PB_Any, 530, 360, 30, 20, "")
strStartY = StringGadget(#PB_Any, 570, 360, 30, 20, "")
strEndX = StringGadget(#PB_Any, 620, 360, 30, 20, "")
strEndY = StringGadget(#PB_Any, 660, 360, 30, 20, "")
strAngle1 = StringGadget(#PB_Any, 530, 420, 30, 20, "")
strAngle2 = StringGadget(#PB_Any, 580, 420, 30, 20, "")
strRadius = StringGadget(#PB_Any, 630, 420, 30, 20, "")

DrawGrid()
  
;Create Image to clear drawing area
CreateImage(#Empty_Image, 500,500, 24)
If StartDrawing(ImageOutput(#Empty_Image))
  DrawImage(GetGadgetAttribute(drgCanvas, #PB_Canvas_Image), 0, 0)
  StopDrawing()
EndIf
If StartDrawing(ImageOutput(#Drawn_Image))
  DrawImage(GetGadgetAttribute(drgCanvas, #PB_Canvas_Image), 0, 0)
  StopDrawing()
EndIf
  
;Only needed when adding a new shape
HideGadget(strNewShape,1)
CurrentOperation.s = "Normal"
  
ShapesDB = OpenShapesDB()
  
If DatabaseQuery(ShapesDB, "SELECT * FROM Groups;")
    
  While NextDatabaseRow(ShapesDB)
      
    AddGadgetItem(cmbGroups, -1, GetDatabaseString(ShapesDB, 1))

  Wend
    
  FinishDatabaseQuery(ShapesDB)

EndIf
  
  Repeat
    Event = WaitWindowEvent()
    
    Select Event
      Case #PB_Event_Menu
        Select EventMenu()
          Case #mnuShapeNew
            NewShape()
          Case #muShapeLoad
          Case #mnuShapeSave
            SaveShape()   
          Case #MenuItem_5
          Case #mnuShapeView
          Case #mnuShapeTest
          Case #MenuItem_8
          Case #mnuShapeExit
            End
            
        EndSelect

   Case #PB_Event_Gadget
    
     Select EventGadget()
          
       Case btnNew
         
         NewShape()         
        
       Case btnLoad 
         
         ;Clear Canvas
         StartVectorDrawing(CanvasVectorOutput(drgCanvas))
         DrawVectorImage(ImageID(#Empty_Image), 255)
         StopVectorDrawing()
         ;DrawGrid()
         
         ;Load the selected shape
         LoadShape(SelectedShape_id)
         
         ;Allow Edit
         CurrentOperation  = "Edit"
         
       Case btnSave
         
         SaveShape()         
                     
       Case btnCancel
          
         ;Reset Current operation
         CurrentOperation  = "Idle"
          
         ;Empty Element array
          
         ;Clear Canvas
         StartVectorDrawing(CanvasVectorOutput(drgCanvas))
         DrawVectorImage(ImageID(#Empty_Image), 255)
         StopVectorDrawing()
          
         ;Redraw drawing area
        ; DrawGrid()
          
         ;Clear new shape string
         SetGadgetText(strNewShape,"")
          
         ;Hide new shape string gadget
         HideGadget(strNewShape,1)
          
         ;Show Shapes combo
         HideGadget(cmbShapes,0)
          
         ;Reset window title
         SetWindowTitle(MainWnd,"Shape Designer")
          
       Case cmbGroups
          
         LoadShapescmb()
           
       Case btn_Line
         
         If StartDrawing(ImageOutput(#Drawn_Image))
           DrawImage(GetGadgetAttribute(drgCanvas, #PB_Canvas_Image), 0, 0)
           StopDrawing()
         EndIf
         CurrentMode = "Line" 
         DrawAction = 0
         
       Case btn_Arc
         
         If StartDrawing(ImageOutput(#Drawn_Image))
           DrawImage(GetGadgetAttribute(drgCanvas, #PB_Canvas_Image), 0, 0)
           StopDrawing()
         EndIf
         CurrentMode = "Arc" 
         DrawAction = 0
           
         ;Clear str gadgets
         SetGadgetText(strAngle1,"")
         SetGadgetText(strAngle2,"")
         SetGadgetText(strRadius,"")
         SetGadgetText(strEndX,"") 
         SetGadgetText(strEndY,"") 
         
       Case cmbShapes

         Shape = GetGadgetText(cmbShapes)
         
         If Shape > ""

           If DatabaseQuery(ShapesDB, "SELECT * FROM Shapes WHERE Title = '" + Shape + "';") 
             
             FirstDatabaseRow(ShapesDB)
             
             SelectedShape_id = GetDatabaseLong(ShapesDB, DatabaseColumnIndex(ShapesDB, "Shape_ID"))
           EndIf
         EndIf   
          
       Case drgCanvas
         
         ;Snap To grid
         X = Round((GetGadgetAttribute(drgCanvas, #PB_Canvas_MouseX)/5) ,#PB_Round_Nearest) * 5
         Y = Round((GetGadgetAttribute(drgCanvas, #PB_Canvas_MouseY)/5),#PB_Round_Nearest) * 5
         If X < 0
           X = 0
         ElseIf X > 500
           X = 500
         EndIf
         If Y < 0
           Y = 0
         ElseIf Y > 500
           Y = 500
         EndIf
         
         ;Show Current Mouse Co-ordinates
         SetGadgetText(strCurrentX,Str(X/5))
         SetGadgetText(strCurrentY,Str(Y/5))
        
         Select EventType()
          
            Case #PB_EventType_LeftButtonDown
              
              ;Grab Canvas Gadget Image
              If StartDrawing(ImageOutput(#Drawn_Image))
                DrawImage(GetGadgetAttribute(drgCanvas, #PB_Canvas_Image), 0, 0)
                StopDrawing()
              EndIf
            
              Select CurrentMode
                  
                Case "Line"
                  
                  Select DrawAction
                      
                    Case 0 ;Set start point
                      
                      DrawAction = DrawAction + 1 
                      
                    Case 1
                      
                      NewObject\EndX = X
                      NewObject\EndY = Y
                      ProcessAction(X, Y)
                      DrawAction = DrawAction + 1 
                      
                  EndSelect
                
                Case "Arc"
                
                  Select DrawAction
                      
                    Case 0 ;Set start point
 
                      DrawAction = DrawAction + 1
                      
                    Case 1 ;Set Radius And Centre Point
                      
                      NewObject\StartX = X
                      NewObject\StartY = Y
                      ProcessAction(X, Y)
                      DrawAction = DrawAction + 1 
                      
                    Case 2 ;Set Start Angle
                      
                      ProcessAction(X, Y)
                      DrawAction = DrawAction + 1  
                      
                    Case 3 ;Set End Angle
                      
                      ProcessAction(X, Y)
                      DrawAction = DrawAction + 1                     
                
                  EndSelect
                  
              EndSelect
              
            Case #PB_EventType_LeftButtonUp
             
              Select CurrentMode
                  
                Case "Line"
                  
                  If DrawAction > 1
                    AddElement(Elements())
                    Elements()\Type = "Line"
                    Elements()\StartX = NewObject\StartX
                    Elements()\StartY = NewObject\StartY
                    Elements()\EndX = NewObject\EndX
                    Elements()\EndY= NewObject\EndY
                    DrawShape()
                    CurrentMode = "Idle"  
                  EndIf   
                   
                Case "Arc"
                   
                  If DrawAction > 3
                    AddElement(Elements())
                    Elements()\Type = "Arc"
                    Elements()\StartX = NewObject\StartX
                    Elements()\StartY = NewObject\StartY
                    Elements()\Radius = NewObject\Radius 
                    Elements()\StartAngle = NewObject\StartAngle
                    Elements()\EndAngle = NewObject\EndAngle
                    DrawShape()
                    CurrentMode = "Idle"  
                  EndIf
                   
                Default
              
                  CurrentMode = "Idle"
                  
              EndSelect
          
            Case #PB_EventType_MouseMove
              
              Select CurrentMode
                  
                Case "Line"
                  
                  SetGadgetText(strEndX,Str(X/5))
                  SetGadgetText(strEndY,Str(Y/5))
                  ProcessAction(X,Y)
                  
                Case "Arc"

                  ProcessAction(X,Y)
                  
              EndSelect
 
            EndSelect
          
          EndSelect
    
     EndSelect
       
  Until Event = #PB_Event_CloseWindow
; IDE Options = PureBasic 5.40 LTS (Windows - x86)
; CursorPosition = 724
; FirstLine = 405
; Folding = A0
; EnableUnicode
; EnableXP